@@ -1,5 +1,5 @@
 /* Tests of renameat.
-   Copyright (C) 2009-2023 Free Software Foundation, Inc.
+   Copyright (C) 2009-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,7 +52,6 @@ do_rename (char const *name1, char const *name2)
 int
 main (void)
 {
-  int i;
   int dfd;
   char *cwd;
   int result;
@@ -63,7 +62,7 @@ main (void)
   /* Test behaviour for invalid file descriptors.  */
   {
     errno = 0;
-    ASSERT (renameat (-1, "foo", AT_FDCWD, "bar") == -1);
+    ASSERT (renameat (AT_FDCWD == -2 ? -1 : -2, "foo", AT_FDCWD, "bar") == -1);
     ASSERT (errno == EBADF);
   }
   {
@@ -75,7 +74,8 @@ main (void)
   ASSERT (close (creat (BASE "oo", 0600)) == 0);
   {
     errno = 0;
-    ASSERT (renameat (AT_FDCWD, BASE "oo", -1, "bar") == -1);
+    ASSERT (renameat (AT_FDCWD, BASE "oo", AT_FDCWD == -2 ? -1 : -2, "bar")
+            == -1);
     ASSERT (errno == EBADF);
   }
   {
@@ -120,7 +120,7 @@ main (void)
      last scenario (two relative paths given, neither one AT_FDCWD)
      has two paths, based on whether the two fds are equivalent, so we
      do the other variant after the loop.  */
-  for (i = 0; i < 16; i++)
+  for (int i = 0; i < 16; i++)
     {
       int fd1 = (i & 8) ? dfd : AT_FDCWD;
       char *file1 = file_name_concat ((i & 4) ? ".." : cwd, BASE "xx", NULL);
@@ -200,8 +200,8 @@ main (void)
   ASSERT (errno == ENOENT);
   free (cwd);
 
-  if (result)
+  if (result == 77)
     fputs ("skipping test: symlinks not supported on this file system\n",
            stderr);
-  return result;
+  return (result ? result : test_exit_status);
 }
