@@ -1,6 +1,6 @@
 /* Map an ino_t inode number to a small integer.
 
-   Copyright 2009-2022 Free Software Foundation, Inc.
+   Copyright 2009-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #include "ino-map.h"
 
 #include "hash.h"
-#include "verify.h"
 
 #include <limits.h>
 #include <stdlib.h>
@@ -60,9 +59,8 @@ ino_hash (void const *x, size_t table_size)
      This avoids loss of info, without applying % to the wider type,
      which could be quite slow on some systems.  */
   size_t h = ino;
-  unsigned int i;
   unsigned int n_words = sizeof ino / sizeof h + (sizeof ino % sizeof h != 0);
-  for (i = 1; i < n_words; i++)
+  for (unsigned int i = 1; i < n_words; i++)
     h ^= ino >> CHAR_BIT * sizeof h * i;
 
   return h % table_size;
@@ -118,8 +116,6 @@ ino_map_free (struct ino_map *map)
 size_t
 ino_map_insert (struct ino_map *im, ino_t ino)
 {
-  struct ino_map_ent *ent;
-
   /* Find space for the probe, reusing the cache if available.  */
   struct ino_map_ent *probe = im->probe;
   if (probe)
@@ -136,7 +132,7 @@ ino_map_insert (struct ino_map *im, ino_t ino)
     }
 
   probe->ino = ino;
-  ent = hash_insert (im->map, probe);
+  struct ino_map_ent *ent = hash_insert (im->map, probe);
   if (! ent)
     return INO_MAP_INSERT_FAILURE;
 
@@ -151,7 +147,7 @@ ino_map_insert (struct ino_map *im, ino_t ino)
          overflow to zero, then it must equal INO_MAP_INSERT_FAILURE,
          which is the value that should be returned in that case.
          Verify that this works.  */
-      verify (INO_MAP_INSERT_FAILURE + 1 == 0);
+      static_assert (INO_MAP_INSERT_FAILURE + 1 == 0);
 
       /* Prepare to allocate a new probe next time; this one is in use.  */
       im->probe = NULL;
