@@ -1,5 +1,5 @@
 /* root-dev-ino.c -- get the device and inode numbers for '/'.
-   Copyright (C) 2003-2023 Free Software Foundation, Inc.
+   Copyright (C) 2003-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,14 +23,14 @@
 #include <stdlib.h>
 
 /* Call lstat to get the device and inode numbers for '/'.
-   Upon failure, return nullptr.  Otherwise, set the members of
+   Upon failure, return NULL.  Otherwise, set the members of
    *ROOT_D_I accordingly and return ROOT_D_I.  */
 struct dev_ino *
 get_root_dev_ino (struct dev_ino *root_d_i)
 {
   struct stat statbuf;
   if (lstat ("/", &statbuf))
-    return nullptr;
+    return NULL;
   root_d_i->st_ino = statbuf.st_ino;
   root_d_i->st_dev = statbuf.st_dev;
   return root_d_i;
