@@ -1,5 +1,5 @@
 /* comm -- compare two sorted files line by line.
-   Copyright (C) 1986-2023 Free Software Foundation, Inc.
+   Copyright (C) 1986-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -36,10 +36,6 @@
   proper_name ("Richard M. Stallman"), \
   proper_name ("David MacKenzie")
 
-/* Undefine, to avoid warning about redefinition on some systems.  */
-#undef min
-#define min(x, y) ((x) < (y) ? (x) : (y))
-
 /* True if the LC_COLLATE locale is hard.  */
 static bool hard_LC_COLLATE;
 
@@ -89,14 +85,14 @@ enum
 
 static struct option const long_options[] =
 {
-  {"check-order", no_argument, nullptr, CHECK_ORDER_OPTION},
-  {"nocheck-order", no_argument, nullptr, NOCHECK_ORDER_OPTION},
-  {"output-delimiter", required_argument, nullptr, OUTPUT_DELIMITER_OPTION},
-  {"total", no_argument, nullptr, TOTAL_OPTION},
-  {"zero-terminated", no_argument, nullptr, 'z'},
+  {"check-order", no_argument, NULL, CHECK_ORDER_OPTION},
+  {"nocheck-order", no_argument, NULL, NOCHECK_ORDER_OPTION},
+  {"output-delimiter", required_argument, NULL, OUTPUT_DELIMITER_OPTION},
+  {"total", no_argument, NULL, TOTAL_OPTION},
+  {"zero-terminated", no_argument, NULL, 'z'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {nullptr, 0, nullptr, 0}
+  {NULL, 0, NULL, 0}
 };
 
 
@@ -124,32 +120,41 @@ With no options, produce three-column output.  Column one contains\n\
 lines unique to FILE1, column two contains lines unique to FILE2,\n\
 and column three contains lines common to both files.\n\
 "), stdout);
+      oputs (_("\
+  -1     suppress column 1 (lines unique to FILE1)\n\
+"));
+      oputs (_("\
+  -2     suppress column 2 (lines unique to FILE2)\n\
+"));
+      oputs (_("\
+  -3     suppress column 3 (lines that appear in both files)\n\
+"));
+      oputs (_("\
+      --check-order\n\
+         check that the input is correctly sorted,\n\
+         even if all input lines are pairable\n\
+"));
+      oputs (_("\
+      --nocheck-order\n\
+         do not check that the input is correctly sorted\n\
+"));
+      oputs (_("\
+      --output-delimiter=STR\n\
+         separate columns with STR\n\
+"));
+      oputs (_("\
+      --total\n\
+         output a summary\n\
+"));
+      oputs (_("\
+  -z, --zero-terminated\n\
+         line delimiter is NUL, not newline\n\
+"));
+      oputs (HELP_OPTION_DESCRIPTION);
+      oputs (VERSION_OPTION_DESCRIPTION);
       fputs (_("\
 \n\
-  -1                      suppress column 1 (lines unique to FILE1)\n\
-  -2                      suppress column 2 (lines unique to FILE2)\n\
-  -3                      suppress column 3 (lines that appear in both files)\n\
-"), stdout);
-      fputs (_("\
-\n\
-      --check-order       check that the input is correctly sorted, even\n\
-                            if all input lines are pairable\n\
-      --nocheck-order     do not check that the input is correctly sorted\n\
-"), stdout);
-      fputs (_("\
-      --output-delimiter=STR  separate columns with STR\n\
-"), stdout);
-      fputs (_("\
-      --total             output a summary\n\
-"), stdout);
-      fputs (_("\
-  -z, --zero-terminated   line delimiter is NUL, not newline\n\
-"), stdout);
-      fputs (HELP_OPTION_DESCRIPTION, stdout);
-      fputs (VERSION_OPTION_DESCRIPTION, stdout);
-      fputs (_("\
-\n\
-Note, comparisons honor the rules specified by 'LC_COLLATE'.\n\
+Comparisons honor the rules specified by 'LC_COLLATE'.\n\
 "), stdout);
       printf (_("\
 \n\
@@ -276,12 +281,10 @@ compare_files (char **infiles)
   /* Counters for the summary.  */
   uintmax_t total[] = {0, 0, 0};
 
-  int i, j;
-
   /* Initialize the storage. */
-  for (i = 0; i < 2; i++)
+  for (int i = 0; i < 2; i++)
     {
-      for (j = 0; j < 4; j++)
+      for (int j = 0; j < 4; j++)
         {
           initbuffer (&lba[i][j]);
           all_line[i][j] = &lba[i][j];
@@ -289,7 +292,7 @@ compare_files (char **infiles)
       alt[i][0] = 0;
       alt[i][1] = 0;
       alt[i][2] = 0;
-      streams[i] = (STREQ (infiles[i], "-") ? stdin : fopen (infiles[i], "r"));
+      streams[i] = (streq (infiles[i], "-") ? stdin : fopen (infiles[i], "r"));
       if (!streams[i])
         error (EXIT_FAILURE, errno, "%s", quotef (infiles[i]));
 
@@ -319,11 +322,10 @@ compare_files (char **infiles)
                               thisline[1]->buffer, thisline[1]->length - 1);
           else
             {
-              size_t len = min (thisline[0]->length, thisline[1]->length) - 1;
+              size_t len = MIN (thisline[0]->length, thisline[1]->length) - 1;
               order = memcmp (thisline[0]->buffer, thisline[1]->buffer, len);
               if (order == 0)
-                order = ((thisline[0]->length > thisline[1]->length)
-                         - (thisline[0]->length < thisline[1]->length));
+                order = _GL_CMP (thisline[0]->length, thisline[1]->length);
             }
         }
 
@@ -358,7 +360,7 @@ compare_files (char **infiles)
       if (order <= 0)
         fill_up[0] = true;
 
-      for (i = 0; i < 2; i++)
+      for (int i = 0; i < 2; i++)
         if (fill_up[i])
           {
             /* Rotate the buffers for this file. */
@@ -386,30 +388,27 @@ compare_files (char **infiles)
           }
     }
 
-  for (i = 0; i < 2; i++)
+  for (int i = 0; i < 2; i++)
     if (fclose (streams[i]) != 0)
       error (EXIT_FAILURE, errno, "%s", quotef (infiles[i]));
 
   if (total_option)
     {
       /* Print the summary, minding the column and line delimiters.  */
-      char buf1[INT_BUFSIZE_BOUND (uintmax_t)];
-      char buf2[INT_BUFSIZE_BOUND (uintmax_t)];
-      char buf3[INT_BUFSIZE_BOUND (uintmax_t)];
       if (col_sep_len == 1)
         { /* Separate to handle NUL char.  */
-          printf ("%s%c%s%c%s%c%s%c",
-                  umaxtostr (total[0], buf1), *col_sep,
-                  umaxtostr (total[1], buf2), *col_sep,
-                  umaxtostr (total[2], buf3), *col_sep,
+          printf ("%ju%c%ju%c%ju%c%s%c",
+                  total[0], *col_sep,
+                  total[1], *col_sep,
+                  total[2], *col_sep,
                   _("total"), delim);
         }
       else
         {
-          printf ("%s%s%s%s%s%s%s%c",
-                  umaxtostr (total[0], buf1), col_sep,
-                  umaxtostr (total[1], buf2), col_sep,
-                  umaxtostr (total[2], buf3), col_sep,
+          printf ("%ju%s%ju%s%ju%s%s%c",
+                  total[0], col_sep,
+                  total[1], col_sep,
+                  total[2], col_sep,
                   _("total"), delim);
         }
     }
@@ -444,7 +443,7 @@ main (int argc, char **argv)
   check_input_order = CHECK_ORDER_DEFAULT;
   total_option = false;
 
-  while ((c = getopt_long (argc, argv, "123z", long_options, nullptr)) != -1)
+  while ((c = getopt_long (argc, argv, "123z", long_options, NULL)) != -1)
     switch (c)
       {
       case '1':
@@ -472,7 +471,7 @@ main (int argc, char **argv)
         break;
 
       case OUTPUT_DELIMITER_OPTION:
-        if (col_sep_len && !STREQ (col_sep, optarg))
+        if (col_sep_len && !streq (col_sep, optarg))
           error (EXIT_FAILURE, 0, _("multiple output delimiters specified"));
         col_sep = optarg;
         col_sep_len = *optarg ? strlen (optarg) : 1;
