@@ -1,5 +1,5 @@
 /* dircolors - output commands to set the LS_COLOR environment variable
-   Copyright (C) 1996-2023 Free Software Foundation, Inc.
+   Copyright (C) 1996-2026 Free Software Foundation, Inc.
    Copyright (C) 1994, 1995, 1997, 1998, 1999, 2000 H. Peter Anvin
 
    This program is free software: you can redistribute it and/or modify
@@ -23,9 +23,8 @@
 
 #include "system.h"
 #include "dircolors.h"
+#include "c-ctype.h"
 #include "c-strcase.h"
-#include "die.h"
-#include "error.h"
 #include "obstack.h"
 #include "quote.h"
 #include "stdio--.h"
@@ -65,9 +64,10 @@ static char const *const ls_codes[] =
 {
   "no", "no", "fi", "rs", "di", "ln", "ln", "ln", "or", "mi", "pi", "pi",
   "so", "bd", "bd", "cd", "cd", "do", "ex", "lc", "lc", "rc", "rc", "ec", "ec",
-  "su", "su", "sg", "sg", "st", "ow", "ow", "tw", "tw", "ca", "mh", "cl", NULL
+  "su", "su", "sg", "sg", "st", "ow", "ow", "tw", "tw", "ca", "mh", "cl",
+  NULL
 };
-static_assert (ARRAY_CARDINALITY (slack_codes) == ARRAY_CARDINALITY (ls_codes));
+static_assert (countof (slack_codes) == countof (ls_codes));
 
 /* Whether to output escaped ls color codes for display.  */
 static bool print_ls_colors;
@@ -104,13 +104,25 @@ usage (int status)
 Output commands to set the LS_COLORS environment variable.\n\
 \n\
 Determine format of output:\n\
-  -b, --sh, --bourne-shell    output Bourne shell code to set LS_COLORS\n\
-  -c, --csh, --c-shell        output C shell code to set LS_COLORS\n\
-  -p, --print-database        output defaults\n\
-      --print-ls-colors       output fully escaped colors for display\n\
 "), stdout);
-      fputs (HELP_OPTION_DESCRIPTION, stdout);
-      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      oputs (_("\
+  -b, --sh, --bourne-shell\n\
+         output Bourne shell code to set LS_COLORS\n\
+"));
+      oputs (_("\
+  -c, --csh, --c-shell\n\
+         output C shell code to set LS_COLORS\n\
+"));
+      oputs (_("\
+  -p, --print-database\n\
+         output defaults\n\
+"));
+      oputs (_("\
+      --print-ls-colors\n\
+         output fully escaped colors for display\n\
+"));
+      oputs (HELP_OPTION_DESCRIPTION);
+      oputs (VERSION_OPTION_DESCRIPTION);
       fputs (_("\
 \n\
 If FILE is specified, read it to determine which colors to use for which\n\
@@ -137,7 +149,7 @@ guess_shell_syntax (void)
 
   shell = last_component (shell);
 
-  if (STREQ (shell, "csh") || STREQ (shell, "tcsh"))
+  if (streq (shell, "csh") || streq (shell, "tcsh"))
     return SHELL_SYNTAX_C;
 
   return SHELL_SYNTAX_BOURNE;
@@ -153,7 +165,7 @@ parse_line (char const *line, char **keyword, char **arg)
   *keyword = NULL;
   *arg = NULL;
 
-  for (p = line; isspace (to_uchar (*p)); ++p)
+  for (p = line; c_isspace (*p); ++p)
     continue;
 
   /* Ignore blank lines and shell-style comments.  */
@@ -162,7 +174,7 @@ parse_line (char const *line, char **keyword, char **arg)
 
   keyword_start = p;
 
-  while (!isspace (to_uchar (*p)) && *p != '\0')
+  while (!c_isspace (*p) && *p != '\0')
     {
       ++p;
     }
@@ -175,7 +187,7 @@ parse_line (char const *line, char **keyword, char **arg)
     {
       ++p;
     }
-  while (isspace (to_uchar (*p)));
+  while (c_isspace (*p));
 
   if (*p == '\0' || *p == '#')
     return;
@@ -185,7 +197,7 @@ parse_line (char const *line, char **keyword, char **arg)
   while (*p != '\0' && *p != '#')
     ++p;
 
-  for (--p; isspace (to_uchar (*p)); --p)
+  for (--p; c_isspace (*p); --p)
     continue;
   ++p;
 
@@ -265,7 +277,7 @@ append_entry (char prefix, char const *item, char const *arg)
 static bool
 dc_parse_stream (FILE *fp, char const *filename)
 {
-  size_t line_number = 0;
+  idx_t line_number = 0;
   char const *next_G_line = G_line;
   char *input_line = NULL;
   size_t input_line_size = 0;
@@ -323,8 +335,8 @@ dc_parse_stream (FILE *fp, char const *filename)
 
       if (arg == NULL)
         {
-          error (0, 0, _("%s:%lu: invalid line;  missing second token"),
-                 quotef (filename), (unsigned long int) line_number);
+          error (0, 0, _("%s:%td: invalid line;  missing second token"),
+                 quotef (filename), line_number);
           ok = false;
           free (keywd);
           continue;
@@ -378,9 +390,9 @@ dc_parse_stream (FILE *fp, char const *filename)
 
       if (unrecognized && (state == ST_TERMSURE || state == ST_TERMYES))
         {
-          error (0, 0, _("%s:%lu: unrecognized keyword %s"),
+          error (0, 0, _("%s:%td: unrecognized keyword %s"),
                  (filename ? quotef (filename) : _("<internal>")),
-                 (unsigned long int) line_number, keywd);
+                 line_number, keywd);
           ok = false;
         }
 
@@ -396,7 +408,7 @@ dc_parse_file (char const *filename)
 {
   bool ok;
 
-  if (! STREQ (filename, "-") && freopen (filename, "r", stdin) == NULL)
+  if (! streq (filename, "-") && freopen (filename, "r", stdin) == NULL)
     {
       error (0, errno, "%s", quotef (filename));
       return false;
@@ -504,10 +516,9 @@ main (int argc, char **argv)
         {
           syntax = guess_shell_syntax ();
           if (syntax == SHELL_SYNTAX_UNKNOWN)
-            {
-              die (EXIT_FAILURE, 0,
-         _("no SHELL environment variable, and no shell type option given"));
-            }
+            error (EXIT_FAILURE, 0,
+                   _("no SHELL environment variable,"
+                     " and no shell type option given"));
         }
 
       obstack_init (&lsc_obstack);
