@@ -1,5 +1,5 @@
 /* env - run a program in a modified environment
-   Copyright (C) 1986-2025 Free Software Foundation, Inc.
+   Copyright (C) 1986-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -87,20 +87,20 @@ enum
 
 static struct option const longopts[] =
 {
-  {"argv0", required_argument, nullptr, 'a'},
-  {"ignore-environment", no_argument, nullptr, 'i'},
-  {"null", no_argument, nullptr, '0'},
-  {"unset", required_argument, nullptr, 'u'},
-  {"chdir", required_argument, nullptr, 'C'},
-  {"default-signal", optional_argument, nullptr, DEFAULT_SIGNAL_OPTION},
-  {"ignore-signal",  optional_argument, nullptr, IGNORE_SIGNAL_OPTION},
-  {"block-signal",   optional_argument, nullptr, BLOCK_SIGNAL_OPTION},
-  {"list-signal-handling", no_argument, nullptr,  LIST_SIGNAL_HANDLING_OPTION},
-  {"debug", no_argument, nullptr, 'v'},
-  {"split-string", required_argument, nullptr, 'S'},
+  {"argv0", required_argument, NULL, 'a'},
+  {"ignore-environment", no_argument, NULL, 'i'},
+  {"null", no_argument, NULL, '0'},
+  {"unset", required_argument, NULL, 'u'},
+  {"chdir", required_argument, NULL, 'C'},
+  {"default-signal", optional_argument, NULL, DEFAULT_SIGNAL_OPTION},
+  {"ignore-signal",  optional_argument, NULL, IGNORE_SIGNAL_OPTION},
+  {"block-signal",   optional_argument, NULL, BLOCK_SIGNAL_OPTION},
+  {"list-signal-handling", no_argument, NULL,  LIST_SIGNAL_HANDLING_OPTION},
+  {"debug", no_argument, NULL, 'v'},
+  {"split-string", required_argument, NULL, 'S'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {nullptr, 0, nullptr, 0}
+  {NULL, 0, NULL, 0}
 };
 
 void
@@ -119,38 +119,53 @@ Set each NAME to VALUE in the environment and run COMMAND.\n\
 
       emit_mandatory_arg_note ();
 
-      fputs (_("\
-  -a, --argv0=ARG      pass ARG as the zeroth argument of COMMAND\n\
-"), stdout);
-      fputs (_("\
-  -i, --ignore-environment  start with an empty environment\n\
-  -0, --null           end each output line with NUL, not newline\n\
-  -u, --unset=NAME     remove variable from the environment\n\
-"), stdout);
-      fputs (_("\
-  -C, --chdir=DIR      change working directory to DIR\n\
-"), stdout);
-      fputs (_("\
-  -S, --split-string=S  process and split S into separate arguments;\n\
-                        used to pass multiple arguments on shebang lines\n\
-"), stdout);
-      fputs (_("\
-      --block-signal[=SIG]    block delivery of SIG signal(s) to COMMAND\n\
-"), stdout);
-      fputs (_("\
-      --default-signal[=SIG]  reset handling of SIG signal(s) to the default\n\
-"), stdout);
-      fputs (_("\
-      --ignore-signal[=SIG]   set handling of SIG signal(s) to do nothing\n\
-"), stdout);
-      fputs (_("\
-      --list-signal-handling  list non default signal handling to stderr\n\
-"), stdout);
-      fputs (_("\
-  -v, --debug          print verbose information for each processing step\n\
-"), stdout);
-      fputs (HELP_OPTION_DESCRIPTION, stdout);
-      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      oputs (_("\
+  -a, --argv0=ARG\n\
+         pass ARG as the zeroth argument of COMMAND\n\
+"));
+      oputs (_("\
+  -i, --ignore-environment\n\
+         start with an empty environment\n\
+"));
+      oputs (_("\
+  -0, --null\n\
+         end each output line with NUL, not newline\n\
+"));
+      oputs (_("\
+  -u, --unset=NAME\n\
+         remove variable from the environment\n\
+"));
+      oputs (_("\
+  -C, --chdir=DIR\n\
+         change working directory to DIR\n\
+"));
+      oputs (_("\
+  -S, --split-string=S\n\
+         process and split S into separate arguments;\n\
+         used to pass multiple arguments on shebang lines\n\
+"));
+      oputs (_("\
+      --block-signal[=SIG]\n\
+         block delivery of SIG signal(s) to COMMAND\n\
+"));
+      oputs (_("\
+      --default-signal[=SIG]\n\
+         reset handling of SIG signal(s) to the default\n\
+"));
+      oputs (_("\
+      --ignore-signal[=SIG]\n\
+         set handling of SIG signal(s) to do nothing\n\
+"));
+      oputs (_("\
+      --list-signal-handling\n\
+         list non default signal handling to standard error\n\
+"));
+      oputs (_("\
+  -v, --debug\n\
+         print verbose information for each processing step\n\
+"));
+      oputs (HELP_OPTION_DESCRIPTION);
+      oputs (VERSION_OPTION_DESCRIPTION);
       fputs (_("\
 \n\
 A mere - implies -i.  If no COMMAND, print the resulting environment.\n\
@@ -188,7 +203,7 @@ unset_envvars (void)
     }
 }
 
-/* Return a pointer to the end of a valid ${VARNAME} string, or nullptr.
+/* Return a pointer to the end of a valid ${VARNAME} string, or NULL.
    'str' should point to the '$' character.
    First letter in VARNAME must be alpha or underscore,
    rest of letters are alnum or underscore.
@@ -206,31 +221,29 @@ scan_varname (char const *str)
         return end;
     }
 
-  return nullptr;
+  return NULL;
 }
 
 /* Return a pointer to a static buffer containing the VARNAME as
    extracted from a '${VARNAME}' string.
    The returned string will be NUL terminated.
    The returned pointer should not be freed.
-   Return nullptr if not a valid ${VARNAME} syntax.  */
+   Return NULL if not a valid ${VARNAME} syntax.  */
 static char *
 extract_varname (char const *str)
 {
-  idx_t i;
-  char const *p;
+  char const *p = scan_varname (str);
 
-  p = scan_varname (str);
   if (!p)
-    return nullptr;
+    return NULL;
 
   /* -2 and +2 (below) account for the '${' prefix.  */
-  i = p - str - 2;
+  idx_t i = p - str - 2;
 
   if (i >= vnlen)
     {
       free (varname);
-      varname = xpalloc (nullptr, &vnlen, i + 1 - vnlen, -1, sizeof *varname);
+      varname = xpalloc (NULL, &vnlen, i + 1 - vnlen, -1, sizeof *varname);
     }
 
   memcpy (varname, str + 2, i);
@@ -359,7 +372,7 @@ build_argv (char const *str, int extra_argc, int *argc)
   ss.half_alloc = extra_argc + 2;
   ss.extra_argc = extra_argc;
   ss.sep = true;
-  ss.argv[ss.argc] = 0;
+  ss.argv[ss.argc] = NULL;
 
   /* In the following loop,
      'break' causes the character 'newc' to be added to *dest,
@@ -498,7 +511,7 @@ build_argv (char const *str, int extra_argc, int *argc)
       argv[1] = "-S-i -C/tmp A=B"
       argv[2] = "foo"
       argv[3] = "bar"
-      argv[4] = nullptr
+      argv[4] = NULL
    This function will modify argv to be:
       argv[0] = "env"
       argv[1] = "-i"
@@ -506,7 +519,7 @@ build_argv (char const *str, int extra_argc, int *argc)
       argv[3] = "A=B"
       argv[4] = "foo"
       argv[5] = "bar"
-      argv[6] = nullptr
+      argv[6] = NULL
    argc will be updated from 4 to 6.
    optind will be reset to 0 to force getopt_long to rescan all arguments.  */
 static void
@@ -542,9 +555,6 @@ parse_split_string (char const *str, int *orig_optind,
 static void
 parse_signal_action_params (char const *arg, bool set_default)
 {
-  char *opt_sig;
-  char *optarg_writable;
-
   if (! arg)
     {
       /* Without an argument, reset all signals.
@@ -555,9 +565,9 @@ parse_signal_action_params (char const *arg, bool set_default)
       return;
     }
 
-  optarg_writable = xstrdup (arg);
+  char *optarg_writable = xstrdup (arg);
 
-  opt_sig = strtok (optarg_writable, ",");
+  char *opt_sig = strtok (optarg_writable, ",");
   while (opt_sig)
     {
       int signum = operand2sig (opt_sig);
@@ -569,7 +579,7 @@ parse_signal_action_params (char const *arg, bool set_default)
 
       signals[signum] = set_default ? DEFAULT : IGNORE;
 
-      opt_sig = strtok (nullptr, ",");
+      opt_sig = strtok (NULL, ",");
     }
 
   free (optarg_writable);
@@ -591,7 +601,7 @@ reset_signal_handlers (void)
       bool set_to_default = (signals[i] == DEFAULT
                              || signals[i] == DEFAULT_NOERR);
 
-      int sig_err = sigaction (i, nullptr, &act);
+      int sig_err = sigaction (i, NULL, &act);
 
       if (sig_err && !ignore_errors)
         error (EXIT_CANCELED, errno,
@@ -600,7 +610,7 @@ reset_signal_handlers (void)
       if (! sig_err)
         {
           act.sa_handler = set_to_default ? SIG_DFL : SIG_IGN;
-          sig_err = sigaction (i, &act, nullptr);
+          sig_err = sigaction (i, &act, NULL);
           if (sig_err && !ignore_errors)
             error (EXIT_CANCELED, errno,
                    _("failed to set signal action for signal %d"), i);
@@ -623,9 +633,6 @@ reset_signal_handlers (void)
 static void
 parse_block_signal_params (char const *arg, bool block)
 {
-  char *opt_sig;
-  char *optarg_writable;
-
   if (! arg)
     {
       /* Without an argument, reset all signals.  */
@@ -644,9 +651,9 @@ parse_block_signal_params (char const *arg, bool block)
   if (! arg)
     return;
 
-  optarg_writable = xstrdup (arg);
+  char *optarg_writable = xstrdup (arg);
 
-  opt_sig = strtok (optarg_writable, ",");
+  char *opt_sig = strtok (optarg_writable, ",");
   while (opt_sig)
     {
       int signum = operand2sig (opt_sig);
@@ -666,7 +673,7 @@ parse_block_signal_params (char const *arg, bool block)
       else
         sigdelset (block ? &unblock_signals : &block_signals, signum);
 
-      opt_sig = strtok (nullptr, ",");
+      opt_sig = strtok (NULL, ",");
     }
 
   free (optarg_writable);
@@ -677,15 +684,16 @@ set_signal_proc_mask (void)
 {
   /* Get the existing signal mask */
   sigset_t set;
-  char const *debug_act;
 
   sigemptyset (&set);
 
-  if (sigprocmask (0, nullptr, &set))
+  if (sigprocmask (0, NULL, &set))
     error (EXIT_CANCELED, errno, _("failed to get signal process mask"));
 
   for (int i = 1; i <= SIGNUM_BOUND; i++)
     {
+      char const *debug_act = NULL;
+
       if (sigismember (&block_signals, i))
         {
           sigaddset (&set, i);
@@ -696,10 +704,6 @@ set_signal_proc_mask (void)
           sigdelset (&set, i);
           debug_act = "UNBLOCK";
         }
-      else
-        {
-          debug_act = nullptr;
-        }
 
       if (dev_debug && debug_act)
         {
@@ -711,7 +715,7 @@ set_signal_proc_mask (void)
         }
     }
 
-  if (sigprocmask (SIG_SETMASK, &set, nullptr))
+  if (sigprocmask (SIG_SETMASK, &set, NULL))
     error (EXIT_CANCELED, errno, _("failed to set signal process mask"));
 }
 
@@ -719,16 +723,15 @@ static void
 list_signal_handling (void)
 {
   sigset_t set;
-  char signame[SIG2STR_MAX];
 
   sigemptyset (&set);
-  if (sigprocmask (0, nullptr, &set))
+  if (sigprocmask (0, NULL, &set))
     error (EXIT_CANCELED, errno, _("failed to get signal process mask"));
 
   for (int i = 1; i <= SIGNUM_BOUND; i++)
     {
       struct sigaction act;
-      if (sigaction (i, nullptr, &act))
+      if (sigaction (i, NULL, &act))
         continue;
 
       char const *ignored = act.sa_handler == SIG_IGN ? "IGNORE" : "";
@@ -738,6 +741,7 @@ list_signal_handling (void)
       if (! *ignored && ! *blocked)
         continue;
 
+      char signame[SIG2STR_MAX];
       if (sig2str (i, signame) != 0)
         snprintf (signame, sizeof signame, "SIG%d", i);
       fprintf (stderr, "%-10s (%2d): %s%s%s\n", signame, i,
@@ -759,11 +763,10 @@ initialize_signals (void)
 int
 main (int argc, char **argv)
 {
-  int optc;
   bool ignore_environment = false;
   bool opt_nul_terminate_output = false;
-  char const *newdir = nullptr;
-  char *argv0 = nullptr;
+  char const *newdir = NULL;
+  char *argv0 = NULL;
 
   initialize_main (&argc, &argv);
   set_program_name (argv[0]);
@@ -776,7 +779,8 @@ main (int argc, char **argv)
 
   initialize_signals ();
 
-  while ((optc = getopt_long (argc, argv, shortopts, longopts, nullptr)) != -1)
+  int optc;
+  while ((optc = getopt_long (argc, argv, shortopts, longopts, NULL)) != -1)
     {
       switch (optc)
         {
@@ -830,7 +834,7 @@ main (int argc, char **argv)
         }
     }
 
-  if (optind < argc && STREQ (argv[optind], "-"))
+  if (optind < argc && streq (argv[optind], "-"))
     {
       ignore_environment = true;
       ++optind;
@@ -839,7 +843,7 @@ main (int argc, char **argv)
   if (ignore_environment)
     {
       devmsg ("cleaning environ\n");
-      static char *dummy_environ[] = { nullptr };
+      static char *dummy_environ[] = { NULL };
       environ = dummy_environ;
     }
   else
