@@ -1,5 +1,5 @@
 /* expand - convert tabs to spaces
-   Copyright (C) 1989-2024 Free Software Foundation, Inc.
+   Copyright (C) 1989-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 
 #include <config.h>
 
-#include <ctype.h>
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -50,11 +49,11 @@ static char const shortopts[] = "it:0::1::2::3::4::5::6::7::8::9::";
 
 static struct option const longopts[] =
 {
-  {"tabs", required_argument, nullptr, 't'},
-  {"initial", no_argument, nullptr, 'i'},
+  {"tabs", required_argument, NULL, 't'},
+  {"initial", no_argument, NULL, 'i'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {nullptr, 0, nullptr, 0}
+  {NULL, 0, NULL, 0}
 };
 
 void
@@ -75,13 +74,17 @@ Convert tabs in each FILE to spaces, writing to standard output.\n\
       emit_stdin_note ();
       emit_mandatory_arg_note ();
 
-      fputs (_("\
-  -i, --initial    do not convert tabs after non blanks\n\
-  -t, --tabs=N     have tabs N characters apart, not 8\n\
-"), stdout);
-      emit_tab_list_info ();
-      fputs (HELP_OPTION_DESCRIPTION, stdout);
-      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      oputs (_("\
+  -i, --initial\n\
+         do not convert tabs after non blanks\n\
+"));
+      oputs (_("\
+  -t, --tabs=N\n\
+         have tabs N characters apart, not 8\n\
+"));
+      emit_tab_list_info (PROGRAM_NAME);
+      oputs (HELP_OPTION_DESCRIPTION);
+      oputs (VERSION_OPTION_DESCRIPTION);
       emit_ancillary_info (PROGRAM_NAME);
     }
   exit (status);
@@ -95,7 +98,7 @@ static void
 expand (void)
 {
   /* Input stream.  */
-  FILE *fp = next_file (nullptr);
+  FILE *fp = next_file (NULL);
 
   if (!fp)
     return;
@@ -113,10 +116,10 @@ expand (void)
          is true:  */
 
       /* Column of next input character.  */
-      uintmax_t column = 0;
+      colno column = 0;
 
       /* Index in TAB_LIST of next tab stop to examine.  */
-      size_t tab_index = 0;
+      idx_t tab_index = 0;
 
 
       /* Convert a line of text.  */
@@ -131,17 +134,9 @@ expand (void)
               if (c == '\t')
                 {
                   /* Column the next input tab stop is on.  */
-                  uintmax_t next_tab_column;
                   bool last_tab;
-
-                  next_tab_column = get_next_tab_column (column, &tab_index,
-                                                         &last_tab);
-
-                  if (last_tab)
-                    next_tab_column = column + 1;
-
-                  if (next_tab_column < column)
-                    error (EXIT_FAILURE, 0, _("input line is too long"));
+                  colno next_tab_column
+                    = get_next_tab_column (column, &tab_index, &last_tab);
 
                   while (++column < next_tab_column)
                     if (putchar (' ') < 0)
@@ -158,8 +153,7 @@ expand (void)
                 }
               else
                 {
-                  column++;
-                  if (!column)
+                  if (ckd_add (&column, column, 1))
                     error (EXIT_FAILURE, 0, _("input line is too long"));
                 }
 
@@ -190,7 +184,7 @@ main (int argc, char **argv)
   atexit (close_stdout);
   convert_entire_line = true;
 
-  while ((c = getopt_long (argc, argv, shortopts, longopts, nullptr)) != -1)
+  while ((c = getopt_long (argc, argv, shortopts, longopts, NULL)) != -1)
     {
       switch (c)
         {
@@ -226,7 +220,7 @@ main (int argc, char **argv)
 
   finalize_tab_stops ();
 
-  set_file_list (optind < argc ? &argv[optind] : nullptr);
+  set_file_list (optind < argc ? &argv[optind] : NULL);
 
   expand ();
 
