@@ -1,5 +1,5 @@
 /* split.c -- split a file into pieces.
-   Copyright (C) 1988-2023 Free Software Foundation, Inc.
+   Copyright (C) 1988-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,12 +21,12 @@
    * support --suppress-matched as in csplit.  */
 #include <config.h>
 
-#include <stdckdint.h>
 #include <stdio.h>
 #include <getopt.h>
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <spawn.h>
 
 #include "system.h"
 #include "alignalloc.h"
@@ -91,7 +91,7 @@ static char const *numeric_suffix_start;
 static char const *additional_suffix;
 
 /* Name of input file.  May be "-".  */
-static char *infile;
+static char const *infile;
 
 /* stat buf for input file.  */
 static struct stat in_stat_buf;
@@ -132,25 +132,25 @@ enum
 
 static struct option const longopts[] =
 {
-  {"bytes", required_argument, nullptr, 'b'},
-  {"lines", required_argument, nullptr, 'l'},
-  {"line-bytes", required_argument, nullptr, 'C'},
-  {"number", required_argument, nullptr, 'n'},
-  {"elide-empty-files", no_argument, nullptr, 'e'},
-  {"unbuffered", no_argument, nullptr, 'u'},
-  {"suffix-length", required_argument, nullptr, 'a'},
-  {"additional-suffix", required_argument, nullptr,
+  {"bytes", required_argument, NULL, 'b'},
+  {"lines", required_argument, NULL, 'l'},
+  {"line-bytes", required_argument, NULL, 'C'},
+  {"number", required_argument, NULL, 'n'},
+  {"elide-empty-files", no_argument, NULL, 'e'},
+  {"unbuffered", no_argument, NULL, 'u'},
+  {"suffix-length", required_argument, NULL, 'a'},
+  {"additional-suffix", required_argument, NULL,
    ADDITIONAL_SUFFIX_OPTION},
-  {"numeric-suffixes", optional_argument, nullptr, 'd'},
-  {"hex-suffixes", optional_argument, nullptr, 'x'},
-  {"filter", required_argument, nullptr, FILTER_OPTION},
-  {"verbose", no_argument, nullptr, VERBOSE_OPTION},
-  {"separator", required_argument, nullptr, 't'},
-  {"-io-blksize", required_argument, nullptr,
+  {"numeric-suffixes", optional_argument, NULL, 'd'},
+  {"hex-suffixes", optional_argument, NULL, 'x'},
+  {"filter", required_argument, NULL, FILTER_OPTION},
+  {"verbose", no_argument, NULL, VERBOSE_OPTION},
+  {"separator", required_argument, NULL, 't'},
+  {"-io-blksize", required_argument, NULL,
    IO_BLKSIZE_OPTION}, /* do not document */
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {nullptr, 0, nullptr, 0}
+  {NULL, 0, NULL, 0}
 };
 
 /* Return true if the errno value, ERR, is ignorable.  */
@@ -181,7 +181,7 @@ set_suffix_length (intmax_t n_units, enum Split_type split_type)
       if (numeric_suffix_start)
         {
           intmax_t n_start;
-          strtol_error e = xstrtoimax (numeric_suffix_start, nullptr, 10,
+          strtol_error e = xstrtoimax (numeric_suffix_start, NULL, 10,
                                        &n_start, "");
           if (e == LONGINT_OK && n_start < n_units)
             {
@@ -234,39 +234,82 @@ default size is 1000 lines, and default PREFIX is 'x'.\n\
       emit_stdin_note ();
       emit_mandatory_arg_note ();
 
-      fprintf (stdout, _("\
-  -a, --suffix-length=N   generate suffixes of length N (default %d)\n\
-      --additional-suffix=SUFFIX  append an additional SUFFIX to file names\n\
-  -b, --bytes=SIZE        put SIZE bytes per output file\n\
-  -C, --line-bytes=SIZE   put at most SIZE bytes of records per output file\n\
-  -d                      use numeric suffixes starting at 0, not alphabetic\n\
-      --numeric-suffixes[=FROM]  same as -d, but allow setting the start value\
-\n\
-  -x                      use hex suffixes starting at 0, not alphabetic\n\
-      --hex-suffixes[=FROM]  same as -x, but allow setting the start value\n\
-  -e, --elide-empty-files  do not generate empty output files with '-n'\n\
-      --filter=COMMAND    write to shell COMMAND; file name is $FILE\n\
-  -l, --lines=NUMBER      put NUMBER lines/records per output file\n\
-  -n, --number=CHUNKS     generate CHUNKS output files; see explanation below\n\
-  -t, --separator=SEP     use SEP instead of newline as the record separator;\n\
-                            '\\0' (zero) specifies the NUL character\n\
-  -u, --unbuffered        immediately copy input to output with '-n r/...'\n\
+      oprintf (_("\
+  -a, --suffix-length=N\n\
+         generate suffixes of length N (default %d)\n\
 "), DEFAULT_SUFFIX_LENGTH);
-      fputs (_("\
-      --verbose           print a diagnostic just before each\n\
-                            output file is opened\n\
-"), stdout);
-      fputs (HELP_OPTION_DESCRIPTION, stdout);
-      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      oputs (_("\
+      --additional-suffix=SUFFIX\n\
+         append an additional SUFFIX to file names\n\
+"));
+      oputs (_("\
+  -b, --bytes=SIZE\n\
+         put SIZE bytes per output file\n\
+"));
+      oputs (_("\
+  -C, --line-bytes=SIZE\n\
+         put at most SIZE bytes of records per output file\n\
+"));
+      oputs (_("\
+  -d\n\
+         use numeric suffixes starting at 0, not alphabetic\n\
+"));
+      oputs (_("\
+      --numeric-suffixes[=FROM]\n\
+         same as -d, but allow setting the start value\n\
+"));
+      oputs (_("\
+  -x\n\
+         use hex suffixes starting at 0, not alphabetic\n\
+"));
+      oputs (_("\
+      --hex-suffixes[=FROM]\n\
+         same as -x, but allow setting the start value\n\
+"));
+      oputs (_("\
+  -e, --elide-empty-files\n\
+         do not generate empty output files with '-n'\n\
+"));
+      oputs (_("\
+      --filter=COMMAND\n\
+         write to shell COMMAND; file name is $FILE\n\
+"));
+      oputs (_("\
+  -l, --lines=NUMBER\n\
+         put NUMBER lines/records per output file\n\
+"));
+      oputs (_("\
+  -n, --number=CHUNKS\n\
+         generate CHUNKS output files; see explanation below\n\
+"));
+      oputs (_("\
+  -t, --separator=SEP\n\
+         use SEP instead of newline as the record separator;\n\
+         '\\0' (zero) specifies the NUL character\n\
+"));
+      oputs (_("\
+  -u, --unbuffered\n\
+         immediately copy input to output with '-n r/...'\n\
+"));
+      oputs (_("\
+      --verbose\n\
+         print a diagnostic just before each output file is opened\n\
+"));
+      oputs (HELP_OPTION_DESCRIPTION);
+      oputs (VERSION_OPTION_DESCRIPTION);
       emit_size_note ();
       fputs (_("\n\
 CHUNKS may be:\n\
   N       split into N files based on size of input\n\
-  K/N     output Kth of N to stdout\n\
+  K/N     output Kth of N to standard output\n\
   l/N     split into N files without splitting lines/records\n\
-  l/K/N   output Kth of N to stdout without splitting lines/records\n\
+  l/K/N   output Kth of N to standard output without splitting lines/records\n\
   r/N     like 'l' but use round robin distribution\n\
-  r/K/N   likewise but only output Kth of N to stdout\n\
+  r/K/N   likewise but only output Kth of N to standard output\n\
+"), stdout);
+      fputs (_("\n\
+-n (except -nr) will buffer to $TMPDIR, defaulting to /tmp,\n\
+if the input size cannot easily be determined.\n\
 "), stdout);
       emit_ancillary_info (PROGRAM_NAME);
     }
@@ -280,7 +323,7 @@ static off_t
 copy_to_tmpfile (int fd, char *buf, idx_t bufsize)
 {
   FILE *tmp;
-  if (!temp_stream (&tmp, nullptr))
+  if (!temp_stream (&tmp, NULL))
     return -1;
   off_t copied = 0;
   off_t r;
@@ -486,63 +529,83 @@ create (char const *name)
       struct stat out_stat_buf;
       if (fstat (fd, &out_stat_buf) != 0)
         error (EXIT_FAILURE, errno, _("failed to stat %s"), quoteaf (name));
-      if (SAME_INODE (in_stat_buf, out_stat_buf))
+      if (psame_inode (&in_stat_buf, &out_stat_buf))
         error (EXIT_FAILURE, 0, _("%s would overwrite input; aborting"),
                quoteaf (name));
-      bool regularish
-        = S_ISREG (out_stat_buf.st_mode) || S_TYPEISSHM (&out_stat_buf);
-      if (! (regularish && out_stat_buf.st_size == 0)
-          && ftruncate (fd, 0) < 0 && regularish)
+      if (ftruncate (fd, 0) < 0
+          && (S_ISREG (out_stat_buf.st_mode) || S_TYPEISSHM (&out_stat_buf)))
         error (EXIT_FAILURE, errno, _("%s: error truncating"), quotef (name));
 
       return fd;
     }
   else
     {
-      int fd_pair[2];
-      pid_t child_pid;
-      char const *shell_prog = getenv ("SHELL");
-      if (shell_prog == nullptr)
-        shell_prog = "/bin/sh";
       if (setenv ("FILE", name, 1) != 0)
         error (EXIT_FAILURE, errno,
                _("failed to set FILE environment variable"));
       if (verbose)
         fprintf (stdout, _("executing with FILE=%s\n"), quotef (name));
+
+      int result;
+      int fd_pair[2];
+      pid_t child_pid;
+
+      posix_spawnattr_t attr;
+      posix_spawn_file_actions_t actions;
+
+      sigset_t set;
+      sigemptyset (&set);
+      if (default_SIGPIPE)
+        sigaddset (&set, SIGPIPE);
+
+      if (   (result = posix_spawnattr_init (&attr))
+          || (result = posix_spawnattr_setflags (&attr,
+                                                 (POSIX_SPAWN_USEVFORK
+                                                  | POSIX_SPAWN_SETSIGDEF)))
+          || (result = posix_spawnattr_setsigdefault (&attr, &set))
+          || (result = posix_spawn_file_actions_init (&actions))
+         )
+        error (EXIT_FAILURE, result, _("posix_spawn initialization failed"));
+
       if (pipe (fd_pair) != 0)
         error (EXIT_FAILURE, errno, _("failed to create pipe"));
-      child_pid = fork ();
-      if (child_pid == 0)
-        {
-          /* This is the child process.  If an error occurs here, the
-             parent will eventually learn about it after doing a wait,
-             at which time it will emit its own error message.  */
-          int j;
-          /* We have to close any pipes that were opened during an
-             earlier call, otherwise this process will be holding a
-             write-pipe that will prevent the earlier process from
-             reading an EOF on the corresponding read-pipe.  */
-          for (j = 0; j < n_open_pipes; ++j)
-            if (close (open_pipes[j]) != 0)
-              error (EXIT_FAILURE, errno, _("closing prior pipe"));
-          if (close (fd_pair[1]))
-            error (EXIT_FAILURE, errno, _("closing output pipe"));
-          if (fd_pair[0] != STDIN_FILENO)
-            {
-              if (dup2 (fd_pair[0], STDIN_FILENO) != STDIN_FILENO)
-                error (EXIT_FAILURE, errno, _("moving input pipe"));
-              if (close (fd_pair[0]) != 0)
-                error (EXIT_FAILURE, errno, _("closing input pipe"));
-            }
-          if (default_SIGPIPE)
-            signal (SIGPIPE, SIG_DFL);
-          execl (shell_prog, last_component (shell_prog), "-c",
-                 filter_command, (char *) nullptr);
-          error (EXIT_FAILURE, errno, _("failed to run command: \"%s -c %s\""),
-                 shell_prog, filter_command);
-        }
-      if (child_pid < 0)
-        error (EXIT_FAILURE, errno, _("fork system call failed"));
+
+      /* We have to close any pipes that were opened during an
+         earlier call, otherwise this process will be holding a
+         write-pipe that will prevent the earlier process from
+         reading an EOF on the corresponding read-pipe.  */
+      for (int i = 0; i < n_open_pipes; ++i)
+        if ((result = posix_spawn_file_actions_addclose (&actions,
+                                                         open_pipes[i])))
+          break;
+
+      if (   result
+          || (result = posix_spawn_file_actions_addclose (&actions, fd_pair[1]))
+          || (fd_pair[0] != STDIN_FILENO
+              && (   (result = posix_spawn_file_actions_adddup2 (&actions,
+                                                                 fd_pair[0],
+                                                                 STDIN_FILENO))
+                  || (result = posix_spawn_file_actions_addclose (&actions,
+                                                                  fd_pair[0]))))
+         )
+        error (EXIT_FAILURE, result, _("posix_spawn setup failed"));
+
+
+      char const *shell_prog = getenv ("SHELL");
+      if (shell_prog == NULL)
+        shell_prog = "/bin/sh";
+      char const *const argv[] = { last_component (shell_prog), "-c",
+                                   filter_command, NULL };
+
+      result = posix_spawn (&child_pid, shell_prog, &actions, &attr,
+                            (char * const *) argv, environ);
+      if (result != 0)
+        error (EXIT_FAILURE, errno, _("failed to run command: \"%s -c %s\""),
+               shell_prog, filter_command);
+
+      posix_spawnattr_destroy (&attr);
+      posix_spawn_file_actions_destroy (&actions);
+
       if (close (fd_pair[0]) != 0)
         error (EXIT_FAILURE, errno, _("failed to close input pipe"));
       filter_pid = child_pid;
@@ -560,14 +623,13 @@ create (char const *name)
 static void
 closeout (FILE *fp, int fd, pid_t pid, char const *name)
 {
-  if (fp != nullptr && fclose (fp) != 0 && ! ignorable (errno))
+  if (fp != NULL && fclose (fp) != 0 && ! ignorable (errno))
     error (EXIT_FAILURE, errno, "%s", quotef (name));
   if (fd >= 0)
     {
-      if (fp == nullptr && close (fd) < 0)
+      if (fp == NULL && close (fd) < 0)
         error (EXIT_FAILURE, errno, "%s", quotef (name));
-      int j;
-      for (j = 0; j < n_open_pipes; ++j)
+      for (int j = 0; j < n_open_pipes; ++j)
         {
           if (open_pipes[j] == fd)
             {
@@ -622,7 +684,7 @@ cwrite (bool new_file_flag, char const *bp, idx_t bytes)
     {
       if (!bp && bytes == 0 && elide_empty_files)
         return true;
-      closeout (nullptr, output_desc, filter_pid, outfile);
+      closeout (NULL, output_desc, filter_pid, outfile);
       next_file_name ();
       output_desc = create (outfile);
       if (output_desc < 0)
@@ -716,7 +778,7 @@ bytes_split (intmax_t n_bytes, intmax_t rem_bytes,
      any existing files or notifies any consumers on fifos.
      FIXME: Should we do this before EXIT_FAILURE?  */
   while (opened++ < max_files)
-    cwrite (true, nullptr, 0);
+    cwrite (true, NULL, 0);
 }
 
 /* Split into pieces of exactly N_LINES lines.
@@ -775,7 +837,7 @@ line_bytes_split (intmax_t n_bytes, char *buf, idx_t bufsize)
   ssize_t n_read;
   intmax_t n_out = 0;      /* for each split.  */
   idx_t n_hold = 0;
-  char *hold = nullptr;        /* for lines > bufsize.  */
+  char *hold = NULL;        /* for lines > bufsize.  */
   idx_t hold_size = 0;
   bool split_line = false;  /* Whether a \n was output in a split.  */
 
@@ -789,7 +851,7 @@ line_bytes_split (intmax_t n_bytes, char *buf, idx_t bufsize)
       while (n_left)
         {
           idx_t split_rest = 0;
-          char *eoc = nullptr;
+          char *eoc = NULL;
           char *eol;
 
           /* Determine End Of Chunk and/or End of Line,
@@ -809,10 +871,7 @@ line_bytes_split (intmax_t n_bytes, char *buf, idx_t bufsize)
             {
               cwrite (n_out == 0, hold, n_hold);
               n_out += n_hold;
-              if (n_hold > bufsize)
-                hold = xirealloc (hold, bufsize);
               n_hold = 0;
-              hold_size = bufsize;
             }
 
           /* Output to eol if present.  */
@@ -983,7 +1042,7 @@ lines_chunk_split (intmax_t k, intmax_t n, char *buf, idx_t bufsize,
               if (chunk_end <= n_written)
                 {
                   if (! k)
-                    cwrite (true, nullptr, 0);
+                    cwrite (true, NULL, 0);
                 }
               else
                 next = false;
@@ -999,7 +1058,7 @@ lines_chunk_split (intmax_t k, intmax_t n, char *buf, idx_t bufsize,
      FIXME: Should we do this before EXIT_FAILURE?  */
   if (!k)
     while (chunk_no++ <= n)
-      cwrite (true, nullptr, 0);
+      cwrite (true, NULL, 0);
 }
 
 /* -n K/N: Extract Kth of N chunks.  */
@@ -1129,7 +1188,7 @@ ofile_open (of_t *files, idx_t i_check, idx_t nfiles)
 
           if (fclose (files[i_reopen].ofile) != 0)
             error (EXIT_FAILURE, errno, "%s", quotef (files[i_reopen].of_name));
-          files[i_reopen].ofile = nullptr;
+          files[i_reopen].ofile = NULL;
           files[i_reopen].ofd = OFD_APPEND;
         }
 
@@ -1159,7 +1218,7 @@ lines_rr (intmax_t k, intmax_t n, char *buf, idx_t bufsize, of_t **filesp)
   bool wrote = false;
   bool file_limit;
   idx_t i_file;
-  of_t *files IF_LINT (= nullptr);
+  of_t *files IF_LINT (= NULL);
   intmax_t line_no;
 
   if (k)
@@ -1176,7 +1235,7 @@ lines_rr (intmax_t k, intmax_t n, char *buf, idx_t bufsize, of_t **filesp)
           next_file_name ();
           files[i_file].of_name = xstrdup (outfile);
           files[i_file].ofd = OFD_NEW;
-          files[i_file].ofile = nullptr;
+          files[i_file].ofile = NULL;
           files[i_file].opid = 0;
         }
       i_file = 0;
@@ -1249,7 +1308,7 @@ lines_rr (intmax_t k, intmax_t n, char *buf, idx_t bufsize, of_t **filesp)
                   if (fclose (files[i_file].ofile) != 0)
                     error (EXIT_FAILURE, errno, "%s",
                            quotef (files[i_file].of_name));
-                  files[i_file].ofile = nullptr;
+                  files[i_file].ofile = NULL;
                   files[i_file].ofd = OFD_APPEND;
                 }
               if (next && ++i_file == n)
@@ -1317,7 +1376,7 @@ static intmax_t
 parse_n_units (char const *arg, char const *multipliers, char const *msgid)
 {
   intmax_t n;
-  if (OVERFLOW_OK < xstrtoimax (arg, nullptr, 10, &n, multipliers) || n < 1)
+  if (OVERFLOW_OK < xstrtoimax (arg, NULL, 10, &n, multipliers) || n < 1)
     strtoint_die (msgid, arg);
   return n;
 }
@@ -1367,8 +1426,8 @@ main (int argc, char **argv)
 
   /* Parse command line options.  */
 
-  infile = bad_cast ("-");
-  outbase = bad_cast ("x");
+  infile = "-";
+  outbase = "x";
 
   while (true)
     {
@@ -1376,7 +1435,7 @@ main (int argc, char **argv)
       int this_optind = optind ? optind : 1;
 
       c = getopt_long (argc, argv, "0123456789C:a:b:del:n:t:ux",
-                       longopts, nullptr);
+                       longopts, NULL);
       if (c == -1)
         break;
 
@@ -1457,7 +1516,7 @@ main (int argc, char **argv)
               error (EXIT_FAILURE, 0, _("empty record separator"));
             if (optarg[1])
               {
-                if (STREQ (optarg, "\\0"))
+                if (streq (optarg, "\\0"))
                   neweol = '\0';
                 else
                   {
@@ -1541,10 +1600,11 @@ main (int argc, char **argv)
           break;
 
         case IO_BLKSIZE_OPTION:
-          in_blk_size = xdectoumax (optarg, 1,
+          in_blk_size = xnumtoumax (optarg, 10, 1,
                                     MIN (SYS_BUFSIZE_MAX,
                                          MIN (IDX_MAX, SIZE_MAX) - 1),
-                                    multipliers, _("invalid IO block size"), 0);
+                                    multipliers, _("invalid IO block size"),
+                                    0, XTOINT_MIN_RANGE);
           break;
 
         case VERBOSE_OPTION:
@@ -1562,7 +1622,8 @@ main (int argc, char **argv)
 
   if (k_units != 0 && filter_command)
     {
-      error (0, 0, _("--filter does not process a chunk extracted to stdout"));
+      error (0, 0, _("--filter does not process a chunk extracted to "
+                     "standard output"));
       usage (EXIT_FAILURE);
     }
 
@@ -1608,7 +1669,7 @@ main (int argc, char **argv)
     }
 
   /* Open the input file.  */
-  if (! STREQ (infile, "-")
+  if (! streq (infile, "-")
       && fd_reopen (STDIN_FILENO, infile, O_RDONLY, 0) < 0)
     error (EXIT_FAILURE, errno, _("cannot open %s for reading"),
            quoteaf (infile));
@@ -1626,7 +1687,7 @@ main (int argc, char **argv)
 
   if (in_blk_size == 0)
     {
-      in_blk_size = io_blksize (in_stat_buf);
+      in_blk_size = io_blksize (&in_stat_buf);
       if (SYS_BUFSIZE_MAX < in_blk_size)
         in_blk_size = SYS_BUFSIZE_MAX;
     }
@@ -1687,13 +1748,13 @@ main (int argc, char **argv)
       }
       break;
 
-    default:
+    case type_undef: default:
       affirm (false);
     }
 
   if (close (STDIN_FILENO) != 0)
     error (EXIT_FAILURE, errno, "%s", quotef (infile));
-  closeout (nullptr, output_desc, filter_pid, outfile);
+  closeout (NULL, output_desc, filter_pid, outfile);
 
   main_exit (EXIT_SUCCESS);
 }
