@@ -1,5 +1,5 @@
 /* tr -- a filter to translate characters
-   Copyright (C) 1991-2025 Free Software Foundation, Inc.
+   Copyright (C) 1991-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,6 @@
 
 #include <config.h>
 
-#include <ctype.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <getopt.h>
@@ -267,13 +266,13 @@ static char xlate[N_CHARS];
 
 static struct option const long_options[] =
 {
-  {"complement", no_argument, nullptr, 'c'},
-  {"delete", no_argument, nullptr, 'd'},
-  {"squeeze-repeats", no_argument, nullptr, 's'},
-  {"truncate-set1", no_argument, nullptr, 't'},
+  {"complement", no_argument, NULL, 'c'},
+  {"delete", no_argument, NULL, 'd'},
+  {"squeeze-repeats", no_argument, NULL, 's'},
+  {"truncate-set1", no_argument, NULL, 't'},
   {GETOPT_HELP_OPTION_DECL},
   {GETOPT_VERSION_OPTION_DECL},
-  {nullptr, 0, nullptr, 0}
+  {NULL, 0, NULL, 0}
 };
 
 void
@@ -292,15 +291,27 @@ Translate, squeeze, and/or delete characters from standard input,\n\
 writing to standard output.  STRING1 and STRING2 specify arrays of\n\
 characters ARRAY1 and ARRAY2 that control the action.\n\
 \n\
-  -c, -C, --complement    use the complement of ARRAY1\n\
-  -d, --delete            delete characters in ARRAY1, do not translate\n\
-  -s, --squeeze-repeats   replace each sequence of a repeated character\n\
-                            that is listed in the last specified ARRAY,\n\
-                            with a single occurrence of that character\n\
-  -t, --truncate-set1     first truncate ARRAY1 to length of ARRAY2\n\
 "), stdout);
-      fputs (HELP_OPTION_DESCRIPTION, stdout);
-      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      oputs (_("\
+  -c, -C, --complement\n\
+         use the complement of ARRAY1\n\
+"));
+      oputs (_("\
+  -d, --delete\n\
+         delete characters in ARRAY1, do not translate\n\
+"));
+      oputs (_("\
+  -s, --squeeze-repeats\n\
+         replace each sequence of a repeated character\n\
+         that is listed in the last specified ARRAY,\n\
+         with a single occurrence of that character\n\
+"));
+      oputs (_("\
+  -t, --truncate-set1\n\
+         first truncate ARRAY1 to length of ARRAY2\n\
+"));
+      oputs (HELP_OPTION_DESCRIPTION);
+      oputs (VERSION_OPTION_DESCRIPTION);
       fputs (_("\
 \n\
 ARRAYs are specified as strings of characters.  Most represent themselves.\n\
@@ -342,8 +353,9 @@ Translation occurs if -d is not given and both STRING1 and STRING2 appear.\n\
 -t is only significant when translating.  ARRAY2 is extended to length of\n\
 ARRAY1 by repeating its last character as necessary.  Excess characters\n\
 of ARRAY2 are ignored.  Character classes expand in unspecified order;\n\
-while translating, [:lower:] and [:upper:] may be used in pairs to\n\
+while translating, '[:lower:]' and '[:upper:]' may be used in pairs to\n\
 specify case conversion.  Squeezing occurs after translation or deletion.\n\
+Arguments like '[...]' should be quoted, to avoid potential shell globbing.\n\
 "), stdout);
       emit_ancillary_info (PROGRAM_NAME);
     }
@@ -543,9 +555,7 @@ ATTRIBUTE_PURE
 static enum Char_class
 look_up_char_class (char const *class_str, size_t len)
 {
-  enum Char_class i;
-
-  for (i = 0; i < countof (char_class_name); i++)
+  for (enum Char_class i = 0; i < countof (char_class_name); i++)
     if (STREQ_LEN (class_str, char_class_name[i], len)
         && strlen (char_class_name[i]) == len)
       return i;
@@ -590,7 +600,7 @@ make_printable_str (char const *s, size_t len)
   for (size_t i = 0; i < len; i++)
     {
       char buf[5];
-      char const *tmp = nullptr;
+      char const *tmp = NULL;
       unsigned char c = s[i];
 
       switch (c)
@@ -642,7 +652,7 @@ static void
 append_normal_char (struct Spec_list *list, unsigned char c)
 {
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = nullptr;
+  new->next = NULL;
   new->type = RE_NORMAL_CHAR;
   new->u.normal_char = c;
   list->tail->next = new;
@@ -670,7 +680,7 @@ append_range (struct Spec_list *list, unsigned char first, unsigned char last)
       return false;
     }
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = nullptr;
+  new->next = NULL;
   new->type = RE_RANGE;
   new->u.range.first_char = first;
   new->u.range.last_char = last;
@@ -692,7 +702,7 @@ append_char_class (struct Spec_list *list,
   if (char_class == CC_NO_CLASS)
     return false;
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = nullptr;
+  new->next = NULL;
   new->type = RE_CHAR_CLASS;
   new->u.char_class = char_class;
   list->tail->next = new;
@@ -710,7 +720,7 @@ append_repeated_char (struct Spec_list *list, unsigned char the_char,
                       count repeat_count)
 {
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = nullptr;
+  new->next = NULL;
   new->type = RE_REPEATED_CHAR;
   new->u.repeated_char.the_repeated_char = the_char;
   new->u.repeated_char.repeat_count = repeat_count;
@@ -732,7 +742,7 @@ append_equiv_class (struct Spec_list *list,
     return false;
 
   struct List_element *new = xmalloc (sizeof *new);
-  new->next = nullptr;
+  new->next = NULL;
   new->type = RE_EQUIV_CLASS;
   new->u.equiv_code = *equiv_class_str;
   list->tail->next = new;
@@ -798,7 +808,7 @@ find_bracketed_repeat (const struct E_string *es, size_t start_idx,
               char const *digit_str = &es->s[start_idx + 2];
               char *d_end;
               if ((xstrtoumax (digit_str, &d_end, *digit_str == '0' ? 8 : 10,
-                               repeat_count, nullptr)
+                               repeat_count, NULL)
                    != LONGINT_OK)
                   || REPEAT_COUNT_MAXIMUM < *repeat_count
                   || digit_str + digit_str_len != d_end)
@@ -1025,7 +1035,7 @@ get_next (struct Spec_list *s, enum Upper_Lower_class *class)
     }
 
   p = s->tail;
-  if (p == nullptr)
+  if (p == NULL)
     return -1;
 
   switch (p->type)
@@ -1140,7 +1150,7 @@ card_of_complement (struct Spec_list *s)
   bool in_set[N_CHARS] = {0};
 
   s->state = BEGIN_STATE;
-  while ((c = get_next (s, nullptr)) != -1)
+  while ((c = get_next (s, NULL)) != -1)
     {
       cardinality -= (!in_set[c]);
       in_set[c] = true;
@@ -1234,14 +1244,13 @@ validate_case_classes (struct Spec_list *s1, struct Spec_list *s2)
 static void
 get_spec_stats (struct Spec_list *s)
 {
-  struct List_element *p;
   count length = 0;
 
   s->n_indefinite_repeats = 0;
   s->has_equiv_class = false;
   s->has_restricted_char_class = false;
   s->has_char_class = false;
-  for (p = s->head->next; p; p = p->next)
+  for (struct List_element *p = s->head->next; p; p = p->next)
     {
       count len = 0;
 
@@ -1329,7 +1338,7 @@ spec_init (struct Spec_list *spec_list)
 {
   struct List_element *new = xmalloc (sizeof *new);
   spec_list->head = spec_list->tail = new;
-  spec_list->head->next = nullptr;
+  spec_list->head->next = NULL;
 }
 
 /* This function makes two passes over the argument string S.  The first
@@ -1412,10 +1421,10 @@ homogeneous_spec_list (struct Spec_list *s)
 
   s->state = BEGIN_STATE;
 
-  if ((b = get_next (s, nullptr)) == -1)
+  if ((b = get_next (s, NULL)) == -1)
     return false;
 
-  while ((c = get_next (s, nullptr)) != -1)
+  while ((c = get_next (s, NULL)) != -1)
     if (c != b)
       return false;
 
@@ -1662,7 +1671,7 @@ set_initialize (struct Spec_list *s, bool complement_this_set, bool *in_set)
   int c;
 
   s->state = BEGIN_STATE;
-  while ((c = get_next (s, nullptr)) != -1)
+  while ((c = get_next (s, NULL)) != -1)
     in_set[c] = true;
   if (complement_this_set)
     for (size_t i = 0; i < N_CHARS; i++)
@@ -1688,7 +1697,7 @@ main (int argc, char **argv)
 
   atexit (close_stdout);
 
-  while ((c = getopt_long (argc, argv, "+AcCdst", long_options, nullptr)) != -1)
+  while ((c = getopt_long (argc, argv, "+AcCdst", long_options, NULL)) != -1)
     {
       switch (c)
         {
@@ -1767,7 +1776,7 @@ main (int argc, char **argv)
         main_exit (EXIT_FAILURE);
     }
   else
-    s2 = nullptr;
+    s2 = NULL;
 
   validate (s1, s2);
 
@@ -1816,7 +1825,7 @@ main (int argc, char **argv)
             {
               if (!in_s1[i])
                 {
-                  int ch = get_next (s2, nullptr);
+                  int ch = get_next (s2, NULL);
                   affirm (ch != -1 || truncate_set1);
                   if (ch == -1)
                     {
