@@ -1,5 +1,5 @@
 /* yes - output a string repeatedly until killed
-   Copyright (C) 1991-2022 Free Software Foundation, Inc.
+   Copyright (C) 1991-2026 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 
 #include "system.h"
 
-#include "error.h"
 #include "full-write.h"
 #include "long-options.h"
 
@@ -48,8 +47,8 @@ Usage: %s [STRING]...\n\
 Repeatedly output a line with all specified STRING(s), or 'y'.\n\
 \n\
 "), stdout);
-      fputs (HELP_OPTION_DESCRIPTION, stdout);
-      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      oputs (HELP_OPTION_DESCRIPTION);
+      oputs (VERSION_OPTION_DESCRIPTION);
       emit_ancillary_info (PROGRAM_NAME);
     }
   exit (status);
@@ -97,6 +96,11 @@ main (int argc, char **argv)
       reuse_operand_strings = false;
     }
 
+#if defined __CHERI__
+  /* Cheri capability bounds do not allow for this.  */
+  reuse_operand_strings = false;
+#endif
+
   /* Fill the buffer with one copy of the output.  If possible, reuse
      the operands strings; this wins when the buffer would be large.  */
   char *buf = reuse_operand_strings ? *operands : xmalloc (bufalloc);
