@@ -1,7 +1,7 @@
 #!/bin/sh
 # cp from 3.16 fails this test
 
-# Copyright (C) 1997-2013 Free Software Foundation, Inc.
+# Copyright (C) 1997-2026 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 # GNU General Public License for more details.
 
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 . "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
 print_ver_ cp
@@ -29,10 +29,8 @@ cd ..
 
 # It should fail with a message something like this:
 #   ./cp: 'a/foo' and 'b/foo' are the same file
-cp -d a/foo b 2>/dev/null
-
 # Fail this test if the exit status is not 1
-test $? = 1 || fail=1
+returns_ 1 cp -d a/foo b 2>/dev/null || fail=1
 
 test "$(cat a/foo)" = $msg || fail=1
 
