@@ -2,7 +2,7 @@
 # Test df's behavior when the mount list cannot be read.
 # This test is skipped on systems that lack LD_PRELOAD support; that's fine.
 
-# Copyright (C) 2012-2025 Free Software Foundation, Inc.
+# Copyright (C) 2012-2026 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,7 +28,8 @@ grep '^#define HAVE_GETMNTENT 1' $CONFIG_HEADER > /dev/null \
       || skip_ "getmntent is not used on this system"
 
 # Simulate "mtab" failure.
-cat > k.c <<EOF || framework_failure_
+# Replace gnulib streq as that is not available here.
+sed 's/streq/0==str''cmp/' > k.c <<EOF || framework_failure_
 #define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,6 +40,35 @@ cat > k.c <<EOF || framework_failure_
 #include <stdarg.h>
 #include <dlfcn.h>
 
+static FILE *(*fopen_func) (const char *, const char *);
+
+FILE *fopen(const char *path, const char *mode)
+{
+
+  /* get reference to original (libc provided) fopen */
+  if (!fopen_func)
+    {
+      fopen_func = (FILE*(*)(const char *, const char *))
+                   dlsym(RTLD_NEXT, "fopen");
+      if (!fopen_func)
+        {
+          fprintf (stderr, "Failed to find fopen()\n");
+          errno = ESRCH;
+          return NULL;
+        }
+    }
+
+  /* Returning ENOENT here will get read_file_system_list()
+     to fall back to using getmntent() below.  */
+  if (streq (path, "/proc/self/mountinfo"))
+    {
+      errno = ENOENT;
+      return NULL;
+    }
+
+  return fopen_func(path, mode);
+}
+
 int open(const char *path, int flags, ...)
 {
   static int (*open_func)(const char *, int, ...);
@@ -56,13 +86,6 @@ int open(const char *path, int flags, ...)
         }
     }
 
-  va_list ap;
-  va_start (ap, flags);
-  mode_t mode = (sizeof (mode_t) < sizeof (int)
-                 ? va_arg (ap, int)
-                 : va_arg (ap, mode_t));
-  va_end (ap);
-
   /* Returning ENOENT here will get read_file_system_list()
      to fall back to using getmntent() below.  */
   if (streq (path, "/proc/self/mountinfo"))
@@ -70,8 +93,15 @@ int open(const char *path, int flags, ...)
       errno = ENOENT;
       return -1;
     }
-  else
-    return open_func(path, flags, mode);
+
+  va_list ap;
+  va_start (ap, flags);
+  mode_t mode = (sizeof (mode_t) < sizeof (int)
+                 ? va_arg (ap, int)
+                 : va_arg (ap, mode_t));
+  va_end (ap);
+
+  return open_func(path, flags, mode);
 }
 
 struct mntent *getmntent (FILE *fp)
@@ -80,7 +110,7 @@ struct mntent *getmntent (FILE *fp)
   static int done = 0;
   if (!done)
     {
-      fclose (fopen ("x", "w"));
+      fclose (fopen_func ("x", "w"));
       ++done;
     }
   /* Now simulate the failure. */
@@ -98,7 +128,7 @@ cleanup_() { unset LD_PRELOAD; }
 export LD_PRELOAD=$LD_PRELOAD:./k.so
 
 # Test if LD_PRELOAD works:
-df 2>/dev/null
+df
 test -f x || skip_ "internal test failure: maybe LD_PRELOAD doesn't work?"
 
 # These tests are supposed to succeed:
