@@ -1,7 +1,7 @@
 #!/bin/sh
 # Ensure that mknod, mkfifo, mkdir -m MODE work with a restrictive umask
 
-# Copyright (C) 2004-2023 Free Software Foundation, Inc.
+# Copyright (C) 2004-2026 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,4 +36,18 @@ mkdir -m 734 f3   || fail=1
 mode=$(ls -dgo f3|cut -b-10)
 test $mode = drwx-wxr-- || test $mode = drwx-wsr-- || fail=1
 
+mknod --mode='ug+rw,o+r' f4 p || fail=1
+mode=$(ls -dgo f4 | cut -b-10)
+test "$mode" = prw-rw-rw- || fail=1
+
+mkfifo --mode='ug+rw,o+r' f5 || fail=1
+mode=$(ls -dgo f5 | cut -b-10)
+test "$mode" = prw-rw-rw- || fail=1
+
+if ! test -g .; then
+  mkdir --mode='ug+rw,o+r' f6 || fail=1
+  mode=$(ls -dgo f6 | cut -b-10)
+  test "$mode" = drwxrwxrwx || fail=1
+fi
+
 Exit $fail
